-- Filename:     multinor.vhd
-- Filetype:     VHDL code
-- Date:         10 januari 2003
-- Update:       13 maart 2011
-- Description:  Een NOR met meerdere ingangen
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.1
-- Copyright:    (c)2003-2011, De Haagse Hogeschool
--
-- Dit is een NOR met meerdere ingangen. Dat is op zich
-- niet zo vreemd, maar deze heeft een door de gebruiker
-- in te stellen breedte. Door middel van de range attribuut
-- en een for .. loop worden de ingangen een voor een afgetast..
-- Noot: merk op dat hier gebruik gemaakt wordt van een zogenaamde
-- generic. Bij instantiering van deze component in een hogere
-- hierarchie wordt default 8 bits gebruik, maar dit kan aangepast
-- worden m.b.v een generic map statement.
-- Bij deze multinor worden drie beschrijvingen (architectures)
-- gegeven, die elk een andere beschrijving van het probleem geeft.
-- De eerste is een logische beschrijving, namelijk iteratief de
-- ingangsbits or-en en daarna inverteren. De tweede itereert weer
-- over alle ingangen met zet een tussenwaarde op 1 als een ingang
-- 1 is, daarna weer inverteren. De laatste maakt gebruik van het
-- feit dat een NOR ook beschreven kan worden als een poort waarvan
-- alle ingangen een 0 moeten zijn om een 1 te leveren (comparator).

library ieee;
use ieee.std_logic_1164.all;
--library work;
--use work.consts.all;

entity multinor is
    generic (tpd : time := 2 ns;
				 n : natural := 8);
    port (
        x : in std_logic_vector(n-1 downto 0);
        q : out std_logic
    );
end multinor;

architecture logic of multinor is
begin
    P0: process (x) is
    -- tijdelijke opslag d.m.v. een variabele
    variable p : std_logic;
    begin
        -- p eerst op 0
        p := '0';
        -- wandel langs alle inputs van x
        for i in n-1 downto 0 loop
				-- doe een logische OR
            p := p or x(i);
        end loop;
        -- hierboven is een OR dus nu inverteren
        q <= not p after tpd;
    end process P0;
end logic;

architecture mux of multinor is
begin
    P0: process (x) is
    -- tijdelijke opslag d.m.v. een variabele
    variable p : std_logic;
    begin
        -- p eerst op 0
        p := '0';
        -- wandel langs alle inputs van x
        for i in x'range loop
				-- als een input 1 is moet p ook 1 worden
            if x(i) = '1' then
                p := '1';
            end if;
        end loop;
        -- hierboven is een OR dus nu inverteren
        q <= not p after tpd;
    end process P0;
end mux;

architecture compare of multinor is
begin
	-- uitgang is 1 als alle ingangen 0 zijn, anders uitgang is 0
	q <= '1' after tpd when (x = (x'range => '0')) else '0' after tpd;
end compare;
